package com.zanxiang.game.back.serve.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.game.back.serve.dao.mapper.GameTencentAppApiRoleRegisterMapper;
import com.zanxiang.game.back.serve.pojo.entity.*;
import com.zanxiang.game.back.serve.pojo.enums.BackStatusEnum;
import com.zanxiang.game.back.serve.service.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Slf4j
@Service
public class GameTencentAppApiRoleRegisterServiceImpl extends ServiceImpl<GameTencentAppApiRoleRegisterMapper, GameTencentAppApiRoleRegister>
implements IGameTencentAppApiRoleRegisterService {

    @Autowired
    private IGameTencentAppApiUserService gameTencentAppApiUserService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean roleRegisterBack(GameTencentAppApiRoleRegister roleRegisterLog) {
        BackStatusEnum backStatus = doCallback(roleRegisterLog);
        return update(new LambdaUpdateWrapper<GameTencentAppApiRoleRegister>()
                .set(GameTencentAppApiRoleRegister::getIsBack, backStatus.getBackStatus())
                .eq(GameTencentAppApiRoleRegister::getId, roleRegisterLog.getId())
        );
    }


    private BackStatusEnum doCallback(GameTencentAppApiRoleRegister roleRegisterLog) {
        GameTencentAppApiUser user = gameTencentAppApiUserService.getOne(new LambdaQueryWrapper<GameTencentAppApiUser>()
                .eq(GameTencentAppApiUser::getGameId, roleRegisterLog.getGameId())
                .eq(GameTencentAppApiUser::getUserId, roleRegisterLog.getUserId())
                .eq(GameTencentAppApiUser::getAdAccountId, roleRegisterLog.getAdAccountId())
                .orderByDesc(GameTencentAppApiUser::getCreateTime)
                .last("limit 1")
        );
        if (user != null) {
            if (BackStatusEnum.NO.getBackStatus().equals(user.getIsBack())) {
                gameTencentAppApiUserService.userBack(user, true);
            }
        }
        return BackStatusEnum.NO;
    }
}
